#include <Keypad.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <esp_now.h>
#include <WiFi.h>

#define ROW_NUM     4 // four rows
#define COLUMN_NUM  4 // four columns

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, -1);

uint8_t broadcastAddress[] = {0x3C, 0x71, 0xBF, 0x0C, 0x45, 0xFC};

typedef struct struct_sent {
  char id;
  char key;   
} struct_sent;

typedef struct struct_receive {
  String state;
  char result;
} struct_receive;

esp_now_peer_info_t peerInfo;

struct_sent info;
struct_receive control;

char keys[ROW_NUM][COLUMN_NUM] = {
  {'1', '2', '3', 'A'},
  {'4', '5', '6', 'B'},
  {'7', '8', '9', 'C'},
  {'*', '0', '#', 'D'}
};

byte pin_rows[ROW_NUM]      = {19, 18, 5, 17}; // GIOP19, GIOP18, GIOP5, GIOP17 connect to the row pins
byte pin_column[COLUMN_NUM] = {16, 4, 0, 2};   // GIOP16, GIOP4, GIOP0, GIOP2 connect to the column pins

Keypad keypad = Keypad( makeKeymap(keys), pin_rows, pin_column, ROW_NUM, COLUMN_NUM );

//global
char remain[9] ={'1','2','3','4','5','6','7','8','9'};
int r_size = 9;
char key;
int animation;
bool is_chosing;
char chosed_value;

String success;
String state;
char result;
bool go_to_next_round;
bool is_ending = false;
int a_score = 0;
int b_score = 0;

// function
void (*resetfunc)(void)=0;

String compare_score(int a,int b){
  if (a>b) return "Winner!";
  if (b>a) return "Loser!";
  return "Draw!";
}

String announce(int x){
  if (x=='A') return "win";
  if (x=='B') return "lose";
  return "draw";
}

void remain_screen(){
  display.clearDisplay();
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor(0, 8);
  display.println("Chosing:");
  display.setCursor(0, 30);
  for (int i=0 ; i < r_size ; i++){
    display.print(remain[i]);
    display.print(" ");
  }
  display.display();   
}

void chosing_screen(char key){
  display.clearDisplay();
  display.setTextSize(4);
  display.setTextColor(WHITE);
  display.setCursor(0, 20);
  display.print(key);
  display.setTextSize(2);
  display.setCursor(80, 20);
  display.print("OK:0");
  display.setCursor(32, 40);
  display.print("return:*");
  display.display();   
}

void waiting_screen(int animation){
  display.clearDisplay();
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor(0, 8);
  display.print("Waiting:");

  if(animation ==0){
    display.print("-");
  }else if(animation ==1){
    display.print("\\");
  }else if(animation ==2){
    display.print("|");
  }else if(animation ==3){
    display.print("/");
  }

  display.setTextSize(2);
  display.setCursor(0, 30);
  for (int i=0 ; i < r_size ; i++){
    display.print(remain[i]);
    display.print(" ");
  }
  display.display();
}

void start_screen(){
  display.clearDisplay();
  display.setTextSize(2);
  display.setTextColor(WHITE);
  display.setCursor(36, 10);
  display.println("Press");
  display.setCursor(52, 25);
  display.println("to");
  display.setCursor(36, 40);
  display.println("Start");
  display.display();       
}

void announce_screen(){
  display.clearDisplay();
  display.setTextSize(2);
  display.setCursor(30,20);
  display.println(announce(result));

  display.setTextSize(1);
//  display.setCursor(30,30);
  display.println("press any to continue");
  display.display();
  
  key = keypad.waitForKey();
}

void announce_final(){
  display.clearDisplay();
  display.setTextSize(2);
  display.setCursor(0,10);
  display.println(compare_score(a_score, b_score));
  display.display();  
}

bool is_in_remain(char key){
  for (int i=0;i<r_size;i++){
    if(key==remain[i]){
      return true;  
    }
  }
  return false;
}

void remove_num(char value){
  bool find_in = false;
  for (int i =0;i<r_size;i++){
    if(value == remain[i]){
      find_in=true;
    }    
    if(find_in && i!=r_size-1){
      remain[i]=remain[i+1];
    }
  }
  remain[r_size-1]='D';
  r_size--;
}

// Callback when data is sent
void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  Serial.print("\rLast Packet Send Status:\t");
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
  if (status ==0){
    success = "Delivery Success :\n)";
  }
  else{
    success = "Delivery Fail :(\n";
  }
}

// Callback when data is received
void OnDataRecv(const uint8_t * mac, const uint8_t *incomingData, int len) {
  memcpy(&control, incomingData, sizeof(control));
  Serial.print("Bytes received: ");
  Serial.println(len);
  Serial.print("state:");
  state = control.state;
  result = control.result;
  Serial.println(state);
  if(state=="start"){
    resetfunc();
  }
  if(state=="next round"){
    go_to_next_round =true;
  }
  if(state=="end"){
    is_ending = true;
  }
  if (result=='A') a_score++;
  if (result=='B') b_score++;
}

void Sending_Data(char value){
  info.key = value;
  while(true){
    esp_err_t result = esp_now_send(broadcastAddress, (uint8_t *) &info, sizeof(info));
    if (result == ESP_OK) {
      Serial.println("Sent with success");
      return;
    }else {
      Serial.println("Error sending the data");
    }
  }
}



void setup() {
  info.id = 'A';
  Serial.begin(115200);
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3D for 128x64
    Serial.println(F("SSD1306 allocation failed"));
    for(;;);
  }

  WiFi.mode(WIFI_STA);
  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }
  esp_now_register_send_cb(OnDataSent);
  memcpy(peerInfo.peer_addr, broadcastAddress, 6);
  peerInfo.channel = 0;  
  peerInfo.encrypt = false;
  if (esp_now_add_peer(&peerInfo) != ESP_OK){
    Serial.println("Failed to add peer");
    return;
  }
  esp_now_register_recv_cb(OnDataRecv);
  
}


void loop() {
  
  go_to_next_round =false;
  is_chosing=true;
  while(is_chosing){
    remain_screen();
    delay(500);
    key =keypad.waitForKey();
    if (is_in_remain(key)) {
      chosed_value =key;
      chosing_screen(key);
      delay(1000); 
      while(true){
        key = keypad.waitForKey();
        if(key == '0'){
          Serial.println(chosed_value);
          //send data
          Sending_Data(chosed_value);
          remove_num(chosed_value);
          is_chosing =false;
          break;
        }else if(key == '*'){
          break;
        }
      }
    }
  }
  animation =0;
  while(true){
    waiting_screen(animation);
    animation = (animation+1)%4;
    // wait to receive data
    if(go_to_next_round || is_ending){
      announce_screen();
      break;   
    }
  }
  
  while(is_ending){
    announce_final();
  }
  
}
