#include <esp_now.h>
#include <WiFi.h>

#define RA_LED 17
#define GA_LED 5
#define RB_LED 18
#define GB_LED 19
#define Button 23

uint8_t broadcastAddress_A[] = {0x3C, 0x61, 0x05, 0x03, 0x69, 0x64};
uint8_t broadcastAddress_B[] = {0x24, 0x6F, 0x28, 0x28, 0x17, 0x1C};
uint8_t broadcastAddress_C[] = {0xE8, 0x68, 0xE7, 0x22, 0xC1, 0x70};

esp_now_peer_info_t peerInfo;

typedef struct struct_recieve {
  char id;
  char key;
}struct_recieve;

struct_recieve myData;

typedef struct struct_send {
  String state;
  char result;
}struct_send;

struct_send control; 

char key_A = 0;
char key_B = 0;

void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status) {
  char macStr[18];
  Serial.print("Packet to: ");
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x",mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.print(macStr);
  Serial.print(" send status:\t");
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
}

void OnDataRecv(const uint8_t * mac_addr, const uint8_t *incomingData, int len) {
  char macStr[18];
  Serial.print("Packet received from: ");
  snprintf(macStr, sizeof(macStr), "%02x:%02x:%02x:%02x:%02x:%02x", mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);
  Serial.println(macStr);
  memcpy(&myData, incomingData, sizeof(myData));
  Serial.printf("Board ID %c: %u bytes\nKey: %c\n", myData.id, len, myData.key);
  if (myData.id == 'A') {
    key_A = myData.key;  
  }
  else {
    key_B = myData.key;  
  }
 }

void(* resetFunc) (void) = 0;

hw_timer_t *timer = NULL;

 void check_button() {
  if (!digitalRead(Button)) {
    resetFunc();
  } 
 }

void OneToMany(String text) {
  Serial.println(text);
  esp_now_send(broadcastAddress_A, (uint8_t *) &control, sizeof(control));
  delay(1);
  esp_now_send(broadcastAddress_B, (uint8_t *) &control, sizeof(control));
  delay(1);
  esp_now_send(broadcastAddress_C, (uint8_t *) &control, sizeof(control));
}

void set_control(String _state, char _result) {
  control.state = _state;
  control.result = _result;
  Serial.println("--- set_control ----");
  Serial.print("state: ");
  Serial.println(control.state);
  Serial.print("result: ");
  Serial.println(control.result);
}

void reset_key() {
  key_A = 0;
  key_B = 0;
}

void wait_key() {
  bool a = true;
  bool b = true;
  digitalWrite(RA_LED, HIGH);
  digitalWrite(RB_LED, HIGH);
  while(a || b) {
    if (key_A && a) {
      digitalWrite(RA_LED, LOW);
      digitalWrite(GA_LED, HIGH);
      a = false;
    }
    if (key_B && b) {
      digitalWrite(RB_LED, LOW);
      digitalWrite(GB_LED, HIGH);
      b = false;
    }
  }
  delay(1500);
  digitalWrite(GA_LED, LOW);
  digitalWrite(GB_LED, LOW);
}

char compare_key() {
  Serial.print("key_A: ");
  Serial.println(key_A);
  Serial.print("key_B: ");
  Serial.println(key_B);
  if (key_A > key_B) return 'A';
  if (key_A < key_B) return 'B';
  return 'D';
}

void setup(){
  pinMode(RA_LED, OUTPUT);
  pinMode(GA_LED, OUTPUT);
  pinMode(RB_LED, OUTPUT);
  pinMode(GB_LED, OUTPUT);
  pinMode(Button, INPUT_PULLUP);
  digitalWrite(RA_LED, LOW);
  digitalWrite(GA_LED, LOW);
  digitalWrite(RB_LED, LOW);
  digitalWrite(GB_LED, LOW);
  Serial.begin(115200);
  WiFi.mode(WIFI_STA);
  if (esp_now_init() != ESP_OK) {
    Serial.println("Error initializing ESP-NOW");
    return;
  }
  
  esp_now_register_send_cb(OnDataSent);

  peerInfo.channel = 0;  
  peerInfo.encrypt = false;  

  memcpy(peerInfo.peer_addr, broadcastAddress_A, 6);
  if (esp_now_add_peer(&peerInfo) != ESP_OK){
    Serial.println("Failed to add peer");
    return;
  }
  memcpy(peerInfo.peer_addr, broadcastAddress_B, 6);
  if (esp_now_add_peer(&peerInfo) != ESP_OK){
    Serial.println("Failed to add peer");
    return;
  }
  memcpy(peerInfo.peer_addr, broadcastAddress_C, 6);
  if (esp_now_add_peer(&peerInfo) != ESP_OK){
    Serial.println("Failed to add peer");
    return;
  }
  
  esp_now_register_recv_cb(OnDataRecv);

  timer = timerBegin(0, 80, true);
  timerAttachInterrupt(timer,&check_button, true);
  timerAlarmWrite(timer, 100000, true);
  timerAlarmEnable(timer);

}

void loop(){
  set_control("start", 'S');
  OneToMany("------ start -------");
  for (int i=1; i<9; i++) {
    delay(100);
    Serial.printf("----- round: %d -----\n", i);
    reset_key();
    wait_key();
    set_control("next round", compare_key());
    OneToMany("---- next round ----");
  }
  Serial.printf("round: %d", 9);
  reset_key();
  wait_key();
  set_control("end", compare_key());
  OneToMany("------- end --------");
  delay(10000);


}
